/*
 * \brief  Tpm session
 * \author Jean-Adrien Domage <jean-adrien.domage@gapfruit.com>
 * \date   2021-06-24
 */

/*
 * Copyright (C) 2021 gapfruit AG
 */

#ifndef _INCLUDE__TPM__SESSION_H_
#define _INCLUDE__TPM__SESSION_H_

#include <base/rpc.h>
#include <base/fixed_stdint.h>
#include <dataspace/capability.h>
#include <session/session.h>


namespace Tpm {

	using namespace Genode;
	class Session;

	class Tpm_error           : public Exception {};
	class Communication_error : public Exception {};

}

class Tpm::Session: public Genode::Session
{
public:

	/*
	 * An spi session consumes four capabilities, one for each tx/rx buffer,
	 * one dataspace capability for the server's session-object allocation, one for the session.
	 */
	enum { CAP_QUOTA = 4 };

	/**
	 * \return c string containing the service name
	 */
	static const char *service_name() { return "Tpm"; }

	/**
	 * duplex_transfer
	 *
	 * Perform a full duplex data transfer to the endpoint slave device using
	 * the spi driver.
	 *
	 * \param     trxn - transfer struct containing relevant buffers
	 * \details   The client provides two buffers, rx_buffer and tx_buffer. tx_buffer
	 *            is used for reading byte that need to be transferred, rx_buffer
	 *            is used to store received bytes. Both buffers needs to be at least
	 *            buffer_size bytes long.
	 * \attention This method is not thread-safe
	 */
	virtual void invoke(uint8_t const  *tx_buffer,
	                    uint8_t        *rx_buffer,
	                    size_t          buffers_size) = 0;

	GENODE_RPC_THROW(Rpc_invoke, void, rpc_invoke,
	                 GENODE_TYPE_LIST(Communication_error, Tpm_error),
	                 size_t);
	GENODE_RPC(Rpc_tx_dataspace, Dataspace_capability, tx_dataspace);
	GENODE_RPC(Rpc_rx_dataspace, Dataspace_capability, rx_dataspace);

	GENODE_RPC_INTERFACE(Rpc_tx_dataspace, Rpc_rx_dataspace, Rpc_invoke);

};

#endif // _INCLUDE__TPM__SESSION_H_
